@@ -20,10 +20,11 @@ import torch
 import numpy as np
 import pandas as pd
 import os
+import scanpy as sc
 import argparse
 
 from scale import SCALE
-from scale.dataset import SingleCellDataset
+from scale.dataset import load_dataset
 from scale.utils import read_labels, cluster_report, estimate_k, binarization
 from scale.plot import plot_embedding
 
@@ -34,7 +35,8 @@ from torch.utils.data import DataLoader
 if __name__ == '__main__':
 
     parser = argparse.ArgumentParser(description='SCALE: Single-Cell ATAC-seq Analysis via Latent feature Extraction')
-    parser.add_argument('--dataset', '-d', type=str, help='input dataset path')
+    parser.add_argument('--data_list', '-d', type=str, nargs='+', default=[])
+    parser.add_argument('--chunk_size', type=int, default=20000)
     parser.add_argument('--n_centroids', '-k', type=int, help='cluster number')
     parser.add_argument('--outdir', '-o', type=str, default='output/', help='Output path')
     parser.add_argument('--verbose', action='store_true', help='Print loss of training process')
@@ -55,7 +57,7 @@ if __name__ == '__main__':
     parser.add_argument('--impute', action='store_true', help='Save the imputed data')
     parser.add_argument('--binary', action='store_true', help='Save binary imputed data')
 #     parser.add_argument('--no_tsne', action='store_true', help='Not save the tsne embedding')
-    parser.add_argument('--emb', type=str, default='UMAP')
+    parser.add_argument('--embed', type=str, default='UMAP')
     parser.add_argument('--reference', '-r', default=None, type=str, help='Reference celltypes')
     parser.add_argument('--transpose', '-t', action='store_true', help='Transpose the input matrix')
 
@@ -73,25 +75,39 @@ if __name__ == '__main__':
         device='cpu'
     batch_size = args.batch_size
 
-    normalizer = MaxAbsScaler()
-    dataset = SingleCellDataset(args.dataset, low=args.low, high=args.high, min_peaks=args.min_peaks,
-                                transpose=args.transpose, transforms=[normalizer.fit_transform])
-    trainloader = DataLoader(dataset, batch_size=batch_size, shuffle=True, drop_last=True)
-    testloader = DataLoader(dataset, batch_size=batch_size, shuffle=False, drop_last=False)
+#     normalizer = MaxAbsScaler()
+#     dataset = SingleCellDataset(args.dataset, low=args.low, high=args.high, min_peaks=args.min_peaks,
+#                                 transpose=args.transpose, transforms=[normalizer.fit_transform])
+#     trainloader = DataLoader(dataset, batch_size=batch_size, shuffle=True, drop_last=True)
+#     testloader = DataLoader(dataset, batch_size=batch_size, shuffle=False, drop_last=False)
+    adata, trainloader, testloader = load_dataset(
+        args.data_list,
+        batch_categories=None, 
+        join='inner', 
+        batch_key='batch', 
+        batch_name='batch',
+        min_genes=200, 
+        min_cells=3, 
+        n_top_genes=2000, 
+        batch_size=64, 
+        chunk_size=args.chunk_size,
+        log=None,
+        transpose=args.transpose,
+    )
 
-    cell_num = dataset.shape[0] 
-    input_dim = dataset.shape[1] 	
+    cell_num = adata.shape[0] 
+    input_dim = adata.shape[1] 	
     
     if args.n_centroids is None:
-        k = min(estimate_k(dataset.data.T), 30)
+        k = min(estimate_k(adata.X.T), 30)
         print('Estimate k {}'.format(k))
     else:
         k = args.n_centroids
     lr = args.lr
-    name = args.dataset.strip('/').split('/')[-1]
+#     name = args.datas.strip('/').split('/')[-1]
     args.min_peaks = int(args.min_peaks) if args.min_peaks >= 1 else args.min_peaks
     
-    outdir = args.outdir
+    outdir = args.outdir+'/'
     if not os.path.exists(outdir):
         os.makedirs(outdir)
         
@@ -116,53 +132,77 @@ if __name__ == '__main__':
                   verbose=args.verbose,
                   device = device,
                   max_iter=args.max_iter,
-                  name=name,
+#                   name=name,
                   outdir=outdir
                    )
-#         torch.save(model.to('cpu').state_dict(), os.path.join(outdir, 'model.pt')) # save model
+        torch.save(model.state_dict(), os.path.join(outdir, 'model.pt')) # save model
     else:
         print('\n## Loading Model: {}\n'.format(args.pretrain))
         model.load_model(args.pretrain)
         model.to(device)
-
+    
     ### output ###
     print('outdir: {}'.format(outdir))
     # 1. latent feature
-    feature = model.encodeBatch(testloader, device=device, out='z')
-    pd.DataFrame(feature, index=dataset.barcode).to_csv(os.path.join(outdir, 'feature.txt'), sep='\t', header=False)
+    adata.obsm['latent'] = model.encodeBatch(testloader, device=device, out='z')
+#     if args.embed: #and adata.shape[0]<1e6:
+#         log.info('Plot {}'.format(args.embed))
+    sc.pp.neighbors(adata, n_neighbors=30, use_rep='latent')
+    sc.tl.leiden(adata)
+    sc.settings.figdir = outdir
+    sc.set_figure_params(dpi=80, figsize=(6,6), fontsize=10)
+    if args.embed == 'UMAP':
+        sc.tl.umap(adata, min_dist=0.1)
+        color = [c for c in ['batch', 'celltype', 'leiden'] if c in adata.obs]
+        sc.pl.umap(adata, color=color, save='.pdf', wspace=0.4, ncols=4)
+    elif args.embed == 'tSNE':
+        sc.tl.tsne(adata)
+        color = [c for c in ['batch', 'celltype', 'leiden'] if c in adata.obs]
+        sc.pl.umap(adata, color=color, save='.pdf', wspace=0.4, ncols=4)
 
-    # 2. cluster assignments
-    pred = model.predict(testloader, device)
-    pd.Series(pred, index=dataset.barcode).to_csv(os.path.join(outdir, 'cluster_assignments.txt'), sep='\t', header=False)
-            
-    # 3. imputed data
-    if args.impute or args.binary:
-        recon_x = model.encodeBatch(testloader, device, out='x', transforms=[normalizer.inverse_transform])
-        if args.binary:
-            import scipy
-            print("Saving binary imputed data")
-            recon_x = binarization(recon_x, dataset.data).T
-            imputed_dir = outdir+'/binary_imputed/'
-            os.makedirs(imputed_dir, exist_ok=True)
-            scipy.io.mmwrite(imputed_dir+'count.mtx', recon_x)
-            pd.Series(dataset.peaks).to_csv(imputed_dir+'peak.txt', sep='\t', index=False, header=None)
-            pd.Series(dataset.barcode).to_csv(imputed_dir+'barcode.txt', sep='\t', index=False, header=None)
-        elif args.impute:
-            print("Saving imputed data")
-            recon_x = pd.DataFrame(recon_x.T, index=dataset.peaks, columns=dataset.barcode)
-            recon_x.to_csv(os.path.join(outdir, 'imputed_data.txt'), sep='\t') 
-        
-#     torch.save(model.to('cpu').state_dict(), os.path.join(outdir, 'model.pt')) # save model
+#     color = [c for c in ['batch', 'celltype', 'leiden'] if c in adata.obs]
+#     sc.pl.umap(adata, color=color, save='.pdf', wspace=0.4, ncols=4)
     
-#     if not args.no_tsne:
-    print("Plotting embedding")
-    if args.reference:
-        ref = pd.read_csv(args.reference, sep='\t', header=None, index_col=0)[1]
-        labels = ref.reindex(dataset.barcode, fill_value='unknown')
-    else:
-        labels = pred
-    plot_embedding(feature, labels, method=args.emb, 
-                   save=os.path.join(outdir, 'emb_{}.pdf'.format(args.emb)), save_emb=os.path.join(outdir, 'emb_{}.txt'.format(args.emb)))
-#         plot_embedding(feature, labels, 
-#                        save=os.path.join(outdir, 'tsne.pdf'), save_emb=os.path.join(outdir, 'tsne.txt'))
+    adata.obsm['impute'] = model.encodeBatch(testloader, device=device, out='x')
+    adata.obsm['binary'] = binarization(adata.obsm['impute'], adata.X)
+    
+    adata.write(outdir+'adata.h5ad', compression='gzip')
+    
+    
+#     pd.DataFrame(feature, index=dataset.barcode).to_csv(os.path.join(outdir, 'feature.txt'), sep='\t', header=False)
+
+#     # 2. cluster assignments
+#     pred = model.predict(testloader, device)
+#     pd.Series(pred, index=dataset.barcode).to_csv(os.path.join(outdir, 'cluster_assignments.txt'), sep='\t', header=False)
+            
+#     # 3. imputed data
+#     if args.impute or args.binary:
+#         recon_x = model.encodeBatch(testloader, device, out='x', transforms=[normalizer.inverse_transform])
+#         if args.binary:
+#             import scipy
+#             print("Saving binary imputed data")
+#             recon_x = binarization(recon_x, dataset.data).T
+#             imputed_dir = outdir+'/binary_imputed/'
+#             os.makedirs(imputed_dir, exist_ok=True)
+#             scipy.io.mmwrite(imputed_dir+'count.mtx', recon_x)
+#             pd.Series(dataset.peaks).to_csv(imputed_dir+'peak.txt', sep='\t', index=False, header=None)
+#             pd.Series(dataset.barcode).to_csv(imputed_dir+'barcode.txt', sep='\t', index=False, header=None)
+#         elif args.impute:
+#             print("Saving imputed data")
+#             recon_x = pd.DataFrame(recon_x.T, index=dataset.peaks, columns=dataset.barcode)
+#             recon_x.to_csv(os.path.join(outdir, 'imputed_data.txt'), sep='\t') 
+        
+    
+    
+# #     if not args.no_tsne:
+#     print("Plotting embedding")
+#     if args.reference:
+#         ref = pd.read_csv(args.reference, sep='\t', header=None, index_col=0)[1]
+#         labels = ref.reindex(dataset.barcode, fill_value='unknown')
+#     else:
+#         labels = pred
+#     plot_embedding(feature, labels, method=args.emb, 
+#                    save=os.path.join(outdir, 'emb_{}.pdf'.format(args.emb)), save_emb=os.path.join(outdir, 'emb_{}.txt'.format(args.emb)))
+# #         plot_embedding(feature, labels, 
+# #                        save=os.path.join(outdir, 'tsne.pdf'), save_emb=os.path.join(outdir, 'tsne.txt'))
         
