@@ -116,7 +116,6 @@ class VAE(nn.Module):
             n = 200,
             max_iter=30000,
             verbose=True,
-            name='',
             patience=100,
             outdir='./'
        ):
@@ -159,10 +158,14 @@ class VAE(nn.Module):
 
     def encodeBatch(self, dataloader, device='cpu', out='z', transforms=None):
         output = []
-        for i, inputs in enumerate(dataloader):
-            x = inputs
+#         indices = []
+#         indices = np.zeros(dataloader.dataset.shape[0])
+#         for i, inputs in enumerate(dataloader):
+        for x in dataloader:
+#             x = inputs
             x = x.view(x.size(0), -1).float().to(device)
             z, mu, logvar = self.encoder(x)
+#             indices.append(idx)
 
             if out == 'z':
                 output.append(z.detach().cpu())
@@ -170,12 +173,15 @@ class VAE(nn.Module):
                 recon_x = self.decoder(z)
                 output.append(recon_x.detach().cpu().data)
             elif out == 'logit':
-                output.append(self.get_gamma(z)[0].cpu().detach())
+                output.append(self.get_gamma(z)[0].cpu().detach().data)
 
+#         indices = torch.cat(output, ).numpy()
+#         print(indices)
         output = torch.cat(output).numpy()
-        if out == 'x':
-            for transform in transforms:
-                output = transform(output)
+#         output[indices] = output
+#         if out == 'x':
+#             for transform in transforms:
+#                 output = transform(output)
         return output
 
 
